from enum import Enum

from pydantic import BaseModel


class Severity(Enum):
    ERROR = "ERROR"  # Mandatory ISTP requirement
    WARNING = "WARNING"  # Recommended practice
    INFO = "INFO"  # Optional or metadata info


class ValidationResult(BaseModel):
    valid: bool
    reference: str
    severity: Severity
    message: str
    target: str = "Global"


class ValidationResultGroup(BaseModel):
    name: str
    rule_reference: str
    severity: Severity
    results: "list[ValidationResult | ValidationResultGroup]"

    def extend(self,
               new_results: "list[ValidationResult | ValidationResultGroup] | ValidationResult | ValidationResultGroup"):
        if isinstance(new_results, list):
            self.results.extend(new_results)
        else:
            self.results.append(new_results)

    def __iter__(self):
        return self.results.__iter__()

    def __len__(self):
        return len(self.results)




import os
from astralint import load_file
import pytest

__HERE__ = os.path.dirname(__file__)


@pytest.fixture
def mms1_asp2_srvy():
    return load_file(os.path.join(__HERE__, "resources/mms1_asp2_srvy_l1b_stat_00000000_v01.cdf"))



from astralint.base.conformance_suite import filter_rules


class MockRule:
    def __init__(self, name):
        self.name = name
        self.reference = f"ref-{name}"


def test_filter_rules():
    rules = [MockRule("rule1"), MockRule("rule2"), MockRule("rule3")]

    # Test select
    selected = filter_rules(rules, select=["rule1", "rule3"], ignore=None)
    assert len(selected) == 2
    assert selected[0].name == "rule1"
    assert selected[1].name == "rule3"

    # Test ignore
    ignored = filter_rules(rules, select=None, ignore=["rule2"])
    assert len(ignored) == 2
    assert ignored[0].name == "rule1"
    assert ignored[1].name == "rule3"

    # Test both select and ignore (should raise an assertion error)
    try:
        filter_rules(rules, select=["rule1"], ignore=["rule2"])
        assert False, "Should have raised an assertion error when both select and ignore are provided."
    except AssertionError:
        pass


def test_filter_rules_with_regex():
    rules = [MockRule("rule1"), MockRule("rule2"), MockRule("rule3")]

    # Test select with regex
    selected = filter_rules(rules, select=["rule[13]"], ignore=None)
    assert len(selected) == 2
    assert selected[0].name == "rule1"
    assert selected[1].name == "rule3"

    # Test ignore with regex
    ignored = filter_rules(rules, select=None, ignore=["rule[23]"])
    assert len(ignored) == 1
    assert ignored[0].name == "rule1"



from typing import Literal, Any

from ...file import File
from ...validation_result import Severity, ValidationResult, ValidationResultGroup
from .base import BaseAssertion, resolve_path


class ContainsKeysAssertion(BaseAssertion):
    check: Literal["contains_keys"] = "contains_keys"
    keys: list[str]

    def single_assertion(self, file: File, path: str, value: Any) -> ValidationResult:
        if isinstance(value, dict):
            missing_keys = [k for k in self.keys if k not in value]
            if missing_keys:
                return ValidationResult(valid=False, reference="", severity=Severity.ERROR,
                                        message=f"Value at path '{path}' is missing keys: {missing_keys}",
                                        target=self.path)
            else:
                return ValidationResult(valid=True, reference="", severity=Severity.INFO,
                                        message=f"Value at path '{path}' contains all required keys.",
                                        target=self.path)
        else:
            return ValidationResult(valid=False, reference="", severity=Severity.ERROR,
                                    message=f"Value at path '{path}' is not an object.", target=self.path)



from cyclopts import App
from pathlib import Path
from rich.console import Console, Group
from rich.panel import Panel

from .base import *
from .reports import report

app = App()


@app.command()
def lint(path: str, suite: str = "ISTP", select: list[str] | None = None, ignore: list[str] | None = None,
         output: str = "console", dest: Path | None = None):
    """Lint the given file or directory against the specified conformance suite.

    Parameters
    ----------
    path : str
        The path to the file or directory to lint.
    suite : str, optional
        The name of the conformance suite to use for linting. Default is "ISTP".
    select : Optional[list[str]], optional
        A list of rule names to include in the linting process. If not provided, all rules in the suite will be used.
    ignore : Optional[list[str]], optional
        A list of rule names to exclude from the linting process. If not provided, no rules will be excluded.
    output: str, optional
        The output format for the report. Default is "console". Other options may include "html", "json", etc.
    dest : str, optional
        The destination file path for the report. If not provided, the report will not be saved to a file.
    """
    checker: ConformanceSuite | None = get_suite(suite)
    if checker:
        if file := load_file(path):
            results = checker.run(file, select=select, ignore=ignore)
            report(results, output=output, dest=dest)
    else:
        raise ValueError(f"Unknown conformance suite '{suite}'. Available suites: {', '.join(list_all_suites())}")


@app.command()
def list_suites(details: bool = False):
    """List all available conformance suites.

    Parameters
    ----------
    details : bool, optional
        If True, display detailed information about each suite. Default is False.
    """
    console = Console()
    suites = list_all_suites()
    suites_panels = []
    for suite in suites:
        s = get_suite(suite)
        if s:
            suites_panels.append(Panel(Group(
                f"[bold]{s.name}[/bold]",
                s.description,
                f"url: {s.url}"
            ), title=suite))

    console.print(Panel(Group(*suites_panels), title="Available Conformance Suites"))


if __name__ == "__main__":
    app()



from astralint.base import DataType
from astralint.base.yaml_rules.assertions.is_type import IsTypeAssertion
from . import *


def test_is_type(mms1_asp2_srvy):
    assertion = IsTypeAssertion(path="variables/.*/data_type", type=DataType.CHAR.value,
                                message="Variable should be a string.")
    r = assertion.evaluate(mms1_asp2_srvy)

